@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.0.4.  */
+/* A Bison parser, made by GNU Bison 3.0.2.  */
 
 /* Bison implementation for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,13 +44,13 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "3.0.4"
+#define YYBISON_VERSION "3.0.2"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
 
 /* Pure parsers.  */
-#define YYPURE 1
+#define YYPURE 2
 
 /* Push parsers.  */
 #define YYPUSH 0
@@ -74,7 +74,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2018 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) Zend Technologies Ltd. (http://www.zend.com)           |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -117,6 +117,22 @@ int ini_parse(void);
 
 #define ZEND_SYSTEM_INI CG(ini_parser_unbuffered_errors)
 
+static int get_int_val(zval *op) {
+	switch (Z_TYPE_P(op)) {
+		case IS_LONG:
+			return Z_LVAL_P(op);
+		case IS_DOUBLE:
+			return (int)Z_DVAL_P(op);
+		case IS_STRING:
+		{
+			int val = atoi(Z_STRVAL_P(op));
+			zend_string_free(Z_STR_P(op));
+			return val;
+		}
+		EMPTY_SWITCH_DEFAULT_CASE()
+	}
+}
+
 /* {{{ zend_ini_do_op()
 */
 static void zend_ini_do_op(char type, zval *result, zval *op1, zval *op2)
@@ -126,22 +142,8 @@ static void zend_ini_do_op(char type, zval *result, zval *op1, zval *op2)
 	int str_len;
 	char str_result[MAX_LENGTH_OF_LONG+1];
 
-	if (IS_LONG == Z_TYPE_P(op1)) {
-		i_op1 = Z_LVAL_P(op1);
-	} else {
-		i_op1 = atoi(Z_STRVAL_P(op1));
-		zend_string_free(Z_STR_P(op1));
-	}
-	if (op2) {
-		if (IS_LONG == Z_TYPE_P(op2)) {
-			i_op2 = Z_LVAL_P(op2);
-		} else {
-			i_op2 = atoi(Z_STRVAL_P(op2));
-			zend_string_free(Z_STR_P(op2));
-		}
-	} else {
-		i_op2 = 0;
-	}
+	i_op1 = get_int_val(op1);
+	i_op2 = op2 ? get_int_val(op2) : 0;
 
 	switch (type) {
 		case '|':
@@ -164,7 +166,7 @@ static void zend_ini_do_op(char type, zval *result, zval *op1, zval *op2)
 			break;
 	}
 
-	str_len = zend_sprintf(str_result, "%d", i_result);
+	str_len = sprintf(str_result, "%d", i_result);
 	ZVAL_NEW_STR(result, zend_string_init(str_result, str_len, ZEND_SYSTEM_INI));
 }
 /* }}} */
@@ -408,22 +410,6 @@ extern int ini_debug;
     END_OF_LINE = 272
   };
 #endif
-/* Tokens.  */
-#define TC_SECTION 258
-#define TC_RAW 259
-#define TC_CONSTANT 260
-#define TC_NUMBER 261
-#define TC_STRING 262
-#define TC_WHITESPACE 263
-#define TC_LABEL 264
-#define TC_OFFSET 265
-#define TC_DOLLAR_CURLY 266
-#define TC_VARNAME 267
-#define TC_QUOTED_STRING 268
-#define BOOL_TRUE 269
-#define BOOL_FALSE 270
-#define NULL_NULL 271
-#define END_OF_LINE 272
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -739,12 +725,12 @@ static const yytype_uint8 yytranslate[] =
   /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   316,   316,   317,   321,   328,   336,   345,   346,   350,
-     351,   355,   356,   357,   358,   359,   363,   364,   368,   369,
-     370,   374,   375,   376,   377,   378,   379,   383,   384,   385,
-     386,   387,   388,   392,   393,   394,   395,   396,   397,   398,
-     402,   406,   407,   408,   409,   410,   414,   415,   416,   417,
-     418
+       0,   318,   318,   319,   323,   330,   338,   347,   348,   352,
+     353,   357,   358,   359,   360,   361,   365,   366,   370,   371,
+     372,   376,   377,   378,   379,   380,   381,   385,   386,   387,
+     388,   389,   390,   394,   395,   396,   397,   398,   399,   400,
+     404,   408,   409,   410,   411,   412,   416,   417,   418,   419,
+     420
 };
 #endif
 
@@ -1391,6 +1377,66 @@ yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
 
         break;
 
+    case 47: /* section_string_or_value  */
+
+      { zval_ini_dtor(&((*yyvaluep))); }
+
+        break;
+
+    case 48: /* string_or_value  */
+
+      { zval_ini_dtor(&((*yyvaluep))); }
+
+        break;
+
+    case 49: /* option_offset  */
+
+      { zval_ini_dtor(&((*yyvaluep))); }
+
+        break;
+
+    case 50: /* encapsed_list  */
+
+      { zval_ini_dtor(&((*yyvaluep))); }
+
+        break;
+
+    case 51: /* var_string_list_section  */
+
+      { zval_ini_dtor(&((*yyvaluep))); }
+
+        break;
+
+    case 52: /* var_string_list  */
+
+      { zval_ini_dtor(&((*yyvaluep))); }
+
+        break;
+
+    case 53: /* expr  */
+
+      { zval_ini_dtor(&((*yyvaluep))); }
+
+        break;
+
+    case 54: /* cfg_var_ref  */
+
+      { zval_ini_dtor(&((*yyvaluep))); }
+
+        break;
+
+    case 55: /* constant_literal  */
+
+      { zval_ini_dtor(&((*yyvaluep))); }
+
+        break;
+
+    case 56: /* constant_string  */
+
+      { zval_ini_dtor(&((*yyvaluep))); }
+
+        break;
+
 
       default:
         break;
